
// 20080925
// Masaomi Hatakeyama
// Brownian Applet
//
// コンパイルは
// カレントディレクトリに jcommon.jar と jfreechart.jar を置いておいて
// Cygwinだと
// javac -classpath "./jcommon.jar;./jfreechart.jar;." BrownianApplet.java
// Macだと
// javac -classpath "jcommon.jar:jfreechart.jar:." BrownianApplet.java
// とする

import java.awt.*; 
import java.applet.*;
import java.awt.event.*;

// JFreeChart 用
// 実行にはJava v1.4以降が必要
import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartFactory;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.chart.ChartPanel;

public class BrownianApplet extends Applet implements Runnable, ActionListener{

	private Thread th = null;
	private Panel panel1=new Panel();	// メインパネル: 粒子表示と文字データとグラフ表示
	private Panel panel2=new Panel();	// JFreeChart用
	private Panel panel3=new Panel();	// ボタン配置用

	private TextArea ta=new TextArea(10,1);
	private PlotArea canvas=new PlotArea(); //描画領域

	private Button button0=new Button("reset");
	private Button button1=new Button("step");
	private Button button2=new Button("run");
	private Button button3=new Button("stop");

	private Label label=new Label("Number of particles(Max:1000)");
	private TextField text=new TextField("1", 3);
	private Checkbox check = new Checkbox("2D");

	// JFreeChart 用
	private TimeSeriesCollection data1;	// 各粒子の位置変化表示用
	private JFreeChart chart1;
	private TimeSeriesCollection data2;	// 平均二乗変位表示用
	private JFreeChart chart2;

	private int run_times = 1;		// run 制御用.
	
	public void init(){

		// Applet全体のレイアウト設定
		setLayout(new BorderLayout());
	        setBackground(Color.white); //下地の色は黄色

		// 上から順に「描画領域」「文字データ領域」「グラフ領域」
		panel1.setLayout(new GridLayout(3,1));

		panel1.add(canvas);
	   //		panel1.add(ta);


		// JFreeChart の設定
		panel2.setLayout(new GridLayout(1,2));

		data1 = new TimeSeriesCollection();
		data2 = new TimeSeriesCollection();
		chart1 = 
		      ChartFactory.createTimeSeriesChart("Position of Each Particle",
                                  "Time",
                                  "Position",
                                  data1,
                                  false,
                                  false,
                                  false);
		chart2 = 
		      ChartFactory.createTimeSeriesChart("Mean Square Displacement",
                                  "Time",
                                  "MSD",
                                  data2,
                                  false,
                                  false,
                                  false);

		canvas.setDataInstance(data1,data2);		// canvas にインスタンスをセットする
								// データの書き出しは canvas 側に任せる

		// パネルに載せる
		ChartPanel cpanel1 = new ChartPanel(chart1);
		ChartPanel cpanel2 = new ChartPanel(chart2);

		panel2.add(cpanel1);
		panel2.add(cpanel2);
		panel1.add(panel2);
		panel1.add(ta);

		// ボタン類設定
		panel3.setLayout(new FlowLayout());
		panel3.add(label);
		panel3.add(text);
		panel3.add(button0);
		panel3.add(button1);
		panel3.add(button2);
		panel3.add(button3);
		panel3.add(check);
		canvas.check2D(check.getState());

		// Listener の登録
		button0.addActionListener(this);
		button1.addActionListener(this);
		button2.addActionListener(this);
		button3.addActionListener(this);

		// Apllet にパネルをのっける
		add(panel1, "Center");
		add(panel3, "North");
	}

	public void start(){
		th = new Thread(this);
		th.start();
	}
	
	public void stop(){
		th = null;
	}
	public void run(){
		Thread thisThread = Thread.currentThread();
		while(th==thisThread&&run_times>0){
			try {
			canvas.step(); 	  //1ステップ分の計算
			canvas.repaint(); //描画
//			ta.setFont(new Font("ＭＳ 明朝",Font.PLAIN,20));
			ta.setFont(new Font("ＭＳ 明朝",Font.BOLD,15));
			ta.setText(canvas.output());
			Thread.sleep(50); //50ms待つ
			} catch (InterruptedException e){}
			run_times--;
		}

	}

	// ボタンを押したときのイベント処理
	public void actionPerformed(ActionEvent e){
		if(e.getSource()==button0){		// reset ボタン
			chart1.fireChartChanged();
			chart2.fireChartChanged();
			canvas.setMolNum(Integer.parseInt(text.getText()));
			canvas.check2D(check.getState());
			canvas.reset();
			canvas.repaint(); //描画
			ta.setFont(new Font("ＭＳ 明朝",Font.BOLD,10));
			ta.setText(canvas.output());
		}else if(e.getSource()==button1){	// step 実行ボタン
			run_times=1;
			start();
		}else if(e.getSource()==button2){	// run 実行ボタン
			run_times=100;
			start();
		}else if(e.getSource()==button3){	// stop ボタン
			run_times=0;
			stop();
		}
	}
}

